<div class="filter-explode filter-body">
  {% import 'includes/filters/filter-variables.j2' as vars %}

  <p class="doc">
    {% with key='Explode-data-filter' %}{% include "includes/help.html" %}{% endwith %}
    Normalise data from "wide" to "narrow" using the headers as labels. This filter
    uses the special <code>+label</code> attribute to identify columns.
  </p>

  {% include 'includes/filters/label-field.html' %}

  {% set name='explode-header-att%02d' % n %}
  <div class="form-group">
    <label for="{{ name }}">
      Attribute for former header text.
    </label>
    <input
        id="{{ name }}"
        class="form-control"
        name="{{ name }}"
        value="{{ recipe.args[name] or 'header' }}"
        pattern="^{{ vars.filters_pattern }}$"
        title="{{ vars.filters_help }}"
        placeholder="{{ vars.filters_placeholder }}"
        />
  </div>

  {% set name='explode-value-att%02d' % n %}
  <div class="form-group">
    <label for="{{ name }}">
      Attribute for original value.
    </label>
    <input
        id="{{ name }}"
        class="form-control"
        name="{{ name }}"
        value="{{ recipe.args[name] or 'value' }}"
        pattern="^{{ vars.filters_pattern }}$"
        title="{{ vars.filters_help }}"
        placeholder="{{ vars.filters_placeholder }}"
        />
  </div>

</div>
