<!DOCTYPE html>
<html lang="en">
  <head>
    <title>[admin] @ Proxy</title>
    {% include "includes/headers.html" %}
  </head>
  <body class="container-fluid">
    {% include "includes/navbar.html" %}
    <header>
        <ul class="breadcrumb">
          <li class="breadcrumb-item"><a href="/admin/">[admin]</a></li>
          <li class="breadcrumb-item active" aria-current="page">Recipes</li>
        </ul>
      <h1>[admin]</h1>
    </header>
    <main>
      {% include "includes/admin-alert.html" %}
      <ul>
        <li><a href="/admin/recipes/">Manage saved recipes</a></li>
      </ul>
    </main>
    {% include "includes/scripts.html" %}
  </body>
</html>

