<div class="filter-cut filter-body">
  {% import 'includes/filters/filter-variables.j2' as vars %}

  <p class="doc">
    {% with key='Cut-columns-filter' %}{% include "includes/help.html" %}{% endwith %}
    Remove columns from your dataset.
  </p>

  {% include 'includes/filters/label-field.html' %}

  {% set name='cut-include-tags%02d' % n %}
  <div class="form-group">
    <label for="{{ name }}">
      Includes: include only these tags (comma-separated)
    </label>
    <input
        id="{{ name }}"
        class="form-control"
        name="{{ name }}"
        value="{{ recipe.args[name] }}"
        pattern="^{{ vars.filters_pattern }}$"
        title="{{ vars.filters_help }}"
        placeholder="{{ vars.filters_placeholder }}"
        />
  </div>

  {% set name='cut-exclude-tags%02d' % n %}
  <div class="form-group">
    <label for="{{ name }}">
      Excludes: include everything <i>but</i> these tags
      (comma-separated).
    </label>
    <input
        id="{{ name }}"
        class="form-control"
        name="{{ name }}"
        value="{{ recipe.args[name] }}"
        pattern="^{{ vars.filters_pattern }}$"
        title="{{ vars.filters_help }}"
        placeholder="{{ vars.filters_placeholder }}"
        />
  </div>

  {% set name='cut-skip-untagged%02d' %n %}
  <div class="checkbox">
    <label>
      <input type="checkbox" name="{{ name }}"{% if recipe.args[name] %} checked="checked"{% endif %} />
      Remove columns without HXL hashtags
    </label>
  </div>

</div>
