<!DOCTYPE html>
<html lang="en">
  <head>
    <title>[admin] Recipes @ Proxy</title>
    {% include "includes/headers.html" %}
  </head>
  <body class="container-fluid">
    {% include "includes/navbar.html" %}
    <header>
        <ul class="breadcrumb">
          <li class="breadcrumb-item"><a href="/admin/">[admin]</a></li>
          <li class="breadcrumb-item active" aria-current="page">Recipes</li>
        </ul>
      <h1>[admin] Recipes</h1>
    </header>
    <main>
      {% include "includes/admin-alert.html" %}
      <table class="table table-striped">
        <thead>
          <tr>
            <th>ID</th>
            <th>Name</th>
            <th>Description</th>
            <th>Created</th>
            <th>Modified</th>
          </tr>
        </thead>
        <tbody>
          {% for recipe in recipes %}
          <tr>
            <td>
              <a href="/admin/recipes/{{ recipe.recipe_id }}/">{{ recipe.recipe_id }}</a>
            </td>
            <td>{{ recipe.name }}</td>
            <td>{{ recipe.description|nonone }}</td>
            <td>{{ recipe.date_created }}</td>
            <td>{{ recipe.date_modified }}</td>
          </tr>
          {% endfor %}
        </tbody>
      </table>
    </main>
    {% include "includes/scripts.html" %}
  </body>
</html>

