<meta charset="utf-8" />
<meta http-equiv="X-UA-Compatible" content="IE=edge" />
<meta name="viewport" content="width=device-width, initial-scale=1" />
<link href="{{ static('bootstrap/css/bootstrap.min.css') }}" rel="stylesheet" />
<link href="{{ static('bootstrap/css/bootstrap-theme.min.css') }}" rel="stylesheet" />
<link href="{{ static('hxl-proxy/hxl-proxy.css') }}" rel="stylesheet" />
<link rel="apple-touch-icon" sizes="57x57" href="{{ static('hxl-proxy/icons/apple-icon-57x57.png') }}" />
<link rel="apple-touch-icon" sizes="60x60" href="{{ static('hxl-proxy/icons/apple-icon-60x60.png') }}" />
<link rel="apple-touch-icon" sizes="72x72" href="{{ static('hxl-proxy/icons/apple-icon-72x72.png') }}" />
<link rel="apple-touch-icon" sizes="76x76" href="{{ static('hxl-proxy/icons/apple-icon-76x76.png') }}" />
<link rel="apple-touch-icon" sizes="114x114" href="{{ static('hxl-proxy/icons/apple-icon-114x114.png') }}" />
<link rel="apple-touch-icon" sizes="120x120" href="{{ static('hxl-proxy/icons/apple-icon-120x120.png') }}" />
<link rel="apple-touch-icon" sizes="144x144" href="{{ static('hxl-proxy/icons/apple-icon-144x144.png') }}" />
<link rel="apple-touch-icon" sizes="152x152" href="{{ static('hxl-proxy/icons/apple-icon-152x152.png') }}" />
<link rel="apple-touch-icon" sizes="180x180" href="{{ static('hxl-proxy/icons/apple-icon-180x180.png') }}" />
<link rel="icon" type="image/png" sizes="192x192"  href="{{ static('hxl-proxy/icons/android-icon-192x192.png') }}" />
<link rel="icon" type="image/png" sizes="32x32" href="{{ static('hxl-proxy/icons/favicon-32x32.png') }}" />
<link rel="icon" type="image/png" sizes="96x96" href="{{ static('hxl-proxy/icons/favicon-96x96.png') }}" />
<link rel="icon" type="image/png" sizes="16x16" href="{{ static('hxl-proxy/icons/favicon-16x16.png') }}" />
<link rel="manifest" href="{{ static('hxl-proxy/icons/manifest.json') }}" />
<meta name="msapplication-TileColor" content="#ffffff" />
<meta name="msapplication-TileImage" content="/ms-icon-144x144.png" />
<meta name="theme-color" content="#ffffff" />
<!-- HTML5 shim and Respond.js for IE8 support of HTML5 elements and media queries -->
<!--[if lt IE 9]>
    <script src="{{ static('compat/html5shiv.min.js') }}"></script>
    <script src="{{ static('compat/respond.min.js') }}"></script>
    <![endif]-->
