body {
    padding-top: 50px;
}

main {
    margin-top: 50px;
    overflow: auto;
}

.messages {
    margin-top: 1rem;
}

label {
    font-weight: normal;
    font-color: #111111;
    font-variant: small-caps;
}

.required label {
    font-weight: bold;
}

.required label:before {
    content: "* ";
}

section {
    margin-top: 2ex;
}

.panel-title a.collapsed:before {
    content: "+ ";
}

.panel-title a:before {
    content: "-";
}

.table-container {
    overflow-x: auto;
}

aside.preview {
    background: #eeeeee;
    overflow: scroll;
}

.side-preview table {
    font-size: 83.333333%;
}

td>.content {
    display: block;
    max-height: 10ex;
    overflow-y: auto;
}

.hxl-headers th {
    background: #cccccc;
    border: 2px white solid;
}

.hxl-tags th {
    background: #eeeeee;
    font-weight: normal;
    font-style: italic;
    font-size: 90%;
    font-family: mono;
    border: 2px white solid;
}

.panel-heading a:after {
    font-family:'Glyphicons Halflings';
    content:"\e114";
    float: right;
    color: grey;
}

.panel-heading a.collapsed:after {
    content:"\e080";
}

.badge {
    width: 6em;
    padding: .5ex;
}

.severity_error {
    background: red;
}

.severity_warning {
    background: yellow;
    color: black;
}

.severity_info {
    background: green;
    color: black;
}

.validation-message {
    font-size: 120%;
}

.icons {
}

.icons a {
    display: inline-block;
    width: 7em;
    height: 7em;
    padding: .25em;
    text-align: center;
}

.icons a img {
    padding: .25em;
    border: 1px solid;
    border-radius: 1em;
    width: 4em;
    height: 4em;
    text-align: center;
}

.icons a span {
    display: block;
    text-align: center;
}

.doc {
    padding: 2ex;
    background: #eeeecc;
    border-radius: 1ex;
    margin-bottom: 3ex;
}

.source-url {
    position: relative;
}

.source-url div {
    width: 100%;
    whitespace: nowrap;
    overflow: hidden;
    text-overflow: ellipsis;
}

#filters {
    max-width: 40rem;
}

aside h2 {
    font-size: 100%;
    font-weight: bold;
    margin-top: 1rem;
}

aside img {
    border: solid 1px #aaaaaa;
    margin: 1ex 0;
}

.nav-tabs {
    margin-top: 1rem;
}

p {
    margin: 1rem 0;
}

/* jquery-sortable */

body.dragging, body.dragging * {
  cursor: move !important;
}

.dragged {
  position: absolute;
  opacity: 0.5;
  z-index: 2000;
}

ol.filter-list {
    list-style: none;
    padding-left: 0;
}

ol.filter-list li.filter {
    position: relative;
    margin-bottom: 2ex;
    margin-left: 2em;
}

ol.filter-list li.placeholder:before {
    position: absolute;
    content: "\21D2";
    margin-left: -1em;
}

ol.filter-list .handle {
    position: absolute;
    left: -2em;;
    top: .4ex;
}

ol.filter-list .filter-name {
    display: inline-block;
    font-weight: bold;
}

ol.filter-list .filter-label {
    display: block;
    font-style: italic;
}

ol.filter-list .filter-delete {
    position: absolute;
    right: 0;
    top: .4ex;
}

.help-link {
    float: right;
}

.form-group .help, .checkbox .help {
    margin-top: 0;
    padding-top: 0;
    font-size: 83.333333%;
    color: #555555;
    font-style: italic;
}

fieldset legend {
    font-size: 120%;
    font-weight: bold;
}

.aggregate {
    padding: 1ex;
    margin: 0 0 1ex 0;
    border: 1px solid #aaaaaa;
    border-radius: 1ex;
}

.recipe-buttons {
    margin-top: 3ex;
}

.filter-list {
    margin-top: 3ex;
    margin-bottom: 3ex;
}

.highlighted {
    background: #F3E5AB;
}
