<script type="text/javascript" src="https://www.dropbox.com/static/api/2/dropins.js" id="dropboxjs" data-app-key="sm6okywskgdecv9"></script>
<script>
  if (typeof Dropbox == 'undefined') {
  $('#cloud-body').prepend('<p class="alert alert-danger">Dropbox chooser unavailable (you may need to disable any ad blockers).</p>');
  }
</script>
<script src="https://davidmegginson.github.io/hdx-chooser/hdx.js"></script>
<script type="text/javascript" src="https://apis.google.com/js/api.js"></script>
