<p class="small">
  &hellip; or pick a dataset from 
  <a href="#" onclick="return hxl_proxy.choosers.hdx('{{ id }}', '{{ submit }}');">HDX</a> 
  or 
  <a href="#" onclick="return hxl_proxy.choosers.dropbox('{{ id }}', '{{ submit }}');">Dropbox</a>
  or
  <a href="#" onclick="return hxl_proxy.choosers.googleDrive('{{ id }}', '{{ submit }}');">Google Drive</a>
</p>
